@@ -26,12 +26,12 @@
 #ifdef HAVE_LIBGPHOTO
 
 #include <string.h>
+#include <strings.h>
 #include <time.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-help.h>
-#include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
@@ -650,9 +650,9 @@ get_icon_from_mime_type (DialogData *data,
 {
 	GdkPixbuf      *pixbuf = NULL;
 	int             icon_size;
-        GnomeIconTheme *icon_theme = gnome_icon_theme_new ();
+        GtkIconTheme   *icon_theme = gtk_icon_theme_get_default ();
+        GtkIconInfo    *icon_info = NULL;
         char           *icon_name;
-        char           *icon_path;
 
         icon_size = get_default_icon_size (data->dialog);
         icon_name = gnome_icon_lookup (icon_theme,
@@ -663,20 +663,17 @@ get_icon_from_mime_type (DialogData *data,
                                        mime_type,
                                        GNOME_ICON_LOOKUP_FLAGS_NONE,
                                        NULL);
-	icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-                                                  icon_name,
-                                                  icon_size,
-                                                  NULL,
-                                                  NULL);
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
+                                            icon_name,
+                                            icon_size,
+                                            0);
         g_free (icon_name);
 
-	if (icon_path != NULL) {
-		 pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
-                g_free (icon_path);
+	if (icon_info != NULL) {
+		pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+		gtk_icon_info_free (icon_info);
 	}
 
-	g_object_unref (icon_theme);
-
 	return pixbuf;
 }
 
@@ -686,7 +683,7 @@ get_mime_type_from_filename (const char*filename)
 {
 	const char *result;
 
-	char *n1 = g_filename_display_name (filename);
+	char *n1 = gnome_vfs_unescape_string_for_display (filename);
 	char *n2 = g_utf8_strdown (n1, -1);
 	char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
 	result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
@@ -749,7 +746,7 @@ load_images_preview__step (AsyncOperationData *aodata,
 		int        width, height;
 		FileData  *fdata;
 
-		pixbuf = gdk_pixbuf_new_from_file (tmp_filename, NULL);
+		pixbuf = gth_pixbuf_new_from_uri (tmp_filename, NULL, 0, 0, NULL);
 		if (pixbuf == NULL)
 			pixbuf = get_mime_type_icon (data, camera_filename);
 
@@ -1063,15 +1060,11 @@ is_valid_filename (const char *name)
 static char*
 get_folder_name (DialogData *data)
 {
-	char *esc_destination;
 	char *destination;
 	char *film_name;
 	char *path;
 
-	esc_destination = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
-	destination = gnome_vfs_unescape_string (esc_destination, "");
-	g_free (esc_destination);
-
+	destination = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
 	eel_gconf_set_path (PREF_PHOTO_IMPORT_DESTINATION, destination);
 
 	if (! dlg_check_folder (GTH_WINDOW (data->browser), destination)) {
@@ -1206,6 +1199,7 @@ save_image (DialogData *data,
 	char       *camera_folder;
 	const char *camera_filename, *local_path;
 	char       *file_uri;
+	char	   *unescaped_local_folder;
 
 	gp_file_new (&file);
 
@@ -1217,8 +1211,10 @@ save_image (DialogData *data,
 			    GP_FILE_TYPE_NORMAL,
 			    file,
 			    data->context);
-
-	file_uri = get_file_name (data, camera_path, local_folder, n);
+	
+	unescaped_local_folder = gnome_vfs_unescape_string (local_folder, "");
+	file_uri = get_file_name (data, camera_path, unescaped_local_folder, n);
+	g_free (unescaped_local_folder);
 
 	/* FIXME: support non-local saving */
 	local_path = get_file_path_from_uri (file_uri);
@@ -1363,7 +1359,7 @@ adjust_orientation__step (AsyncOperationData *aodata,
 		FileData     *fd = file_data_new (filepath, &info);
 		GthTransform transform = read_orientation_field (fd->path);
 		if (image_is_jpeg (filepath))
-			apply_transformation_jpeg (window, fd->path, transform, FALSE);
+			apply_transformation_jpeg (window, fd->path, transform);
 		else
 			apply_transformation_generic (window, fd->path, transform);
 
@@ -1519,7 +1515,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! ensure_dir_exists (data->local_folder, 0755)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_display_name (data->local_folder);
+		utf8_path = gnome_vfs_unescape_string_for_display (data->local_folder);
 		msg = g_strdup_printf (_("Could not create the folder \"%s\": %s"),
 				       utf8_path,
 				       errno_to_string ());
@@ -1536,7 +1532,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! check_permissions (data->local_folder, R_OK | W_OK | X_OK)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_display_name (data->local_folder);
+		utf8_path = gnome_vfs_unescape_string_for_display (data->local_folder);
 		msg = g_strdup_printf (_("You don't have the right permissions to create images in the folder \"%s\""), utf8_path);
 
 		display_error_dialog (data, _("Could not import photos"), msg);
@@ -1569,7 +1565,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (get_dest_free_space (data->local_folder) < total_size) {
 		display_error_dialog (data,
 				      _("Could not import photos"),
-				      "Not enough free space left on disk"); /* FIXME: mark for translation. */
+				      _("Not enough free space left on disk"));
 
 		g_free (data->local_folder);
 		data->local_folder = NULL;
@@ -1795,7 +1791,6 @@ dlg_photo_importer (GthBrowser *browser)
 	GtkWidget    *btn_help;
 	GdkPixbuf    *mute_pixbuf;
 	char         *default_path;
-	char         *esc_uri;
 	char         *default_film_name;
 
 	data = g_new0 (DialogData, 1);
@@ -1869,8 +1864,8 @@ dlg_photo_importer (GthBrowser *browser)
 
 	/* Set widgets data. */
 
-	data->camera_present_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" CAMERA_FILE, NULL);
-	mute_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" MUTE_FILE, NULL);
+	data->camera_present_pixbuf = gth_pixbuf_new_from_uri (GTHUMB_GLADEDIR "/" CAMERA_FILE, NULL, 0, 0, NULL);
+	mute_pixbuf = gth_pixbuf_new_from_uri (GTHUMB_GLADEDIR "/" MUTE_FILE, NULL, 0, 0, NULL);
 
 	data->no_camera_pixbuf = gdk_pixbuf_copy (data->camera_present_pixbuf);
 	gdk_pixbuf_composite (mute_pixbuf,
@@ -1896,9 +1891,7 @@ dlg_photo_importer (GthBrowser *browser)
 	default_path = eel_gconf_get_path (PREF_PHOTO_IMPORT_DESTINATION, NULL);
 	if ((default_path == NULL) || (*default_path == 0))
 		default_path = g_strdup (g_get_home_dir());
-	esc_uri = gnome_vfs_escape_host_and_path_string (default_path);
-	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton), esc_uri);
-	g_free (esc_uri);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton), default_path);
 	g_free (default_path);
 
 	default_film_name = eel_gconf_get_path (PREF_PHOTO_IMPORT_FILM, "");
