@@ -32,6 +32,7 @@
 #include "comments.h"
 #include "main.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "gconf-utils.h"
 #include "gtk-utils.h"
 #include "gth-browser.h"
@@ -104,8 +105,8 @@ static const GOptionEntry options[] = {
 	  0 },
 
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining_args,
-          NULL,
-          NULL },
+	  NULL,
+	  NULL },
 
 	{ NULL }
 };
@@ -371,6 +372,45 @@ session_is_restored (void)
 }
 
 
+static char *
+get_command_line_catalog_path (void)
+{
+	char *catalog_name_utf8;
+	char *catalog_name;
+	char *catalog_path;
+
+	catalog_name_utf8 = g_strconcat (_("Command Line"),
+					 CATALOG_EXT,
+					 NULL);
+	catalog_name = gnome_vfs_escape_string (catalog_name_utf8);
+	catalog_path = get_catalog_full_path (catalog_name);
+	g_free (catalog_name);
+	g_free (catalog_name_utf8);
+
+	return catalog_path;
+}
+
+
+static void
+reset_command_line_catalog (void)
+{
+	char *catalog_path;
+	char *catalog_uri;
+
+	/* Reset the startup location if it was set to the command
+	 * line catalog. */
+
+	catalog_path = get_command_line_catalog_path ();
+	catalog_uri = g_strconcat ("catalog://", remove_scheme_from_uri (catalog_path), NULL);
+
+	if (strcmp (catalog_uri, preferences_get_startup_location ()) == 0)
+		preferences_set_startup_location (get_home_uri ());
+
+	g_free (catalog_uri);
+	g_free (catalog_path);
+}
+
+
 /* Initialize application data. */
 static void
 initialize_data (void)
@@ -408,8 +448,10 @@ initialize_data (void)
 
 	/* Parse command line arguments. */
 
-	if (remaining_args == NULL) /* No arguments specified. */
+	if (remaining_args == NULL) { /* No arguments specified. */
+		reset_command_line_catalog ();
 		return;
+	}
 
 	current_dir = g_get_current_dir ();
 	while ((filename = remaining_args[i++]) != NULL) {
@@ -443,26 +485,17 @@ initialize_data (void)
 	n_file_urls = g_list_length (file_urls);
 	n_dir_urls = g_list_length (dir_urls);
 
-	if (n_file_urls == 1) {
+	if (n_file_urls == 1)
 		view_single_image = TRUE;
 
-	} else if (n_file_urls > 1) {
+	if (n_file_urls > 1) {
 		/* Create a catalog with the command line list. */
 		Catalog *catalog;
 		char    *catalog_path;
-		char    *catalog_name, *catalog_name_utf8;
 		GList   *scan;
 
 		catalog = catalog_new ();
-
-		catalog_name_utf8 = g_strconcat (_("Command Line"),
-						 CATALOG_EXT,
-						 NULL);
-		catalog_name = gnome_vfs_escape_string (catalog_name_utf8);
-		catalog_path = get_catalog_full_path (catalog_name);
-		g_free (catalog_name);
-		g_free (catalog_name_utf8);
-
+		catalog_path = get_command_line_catalog_path ();
 		catalog_set_path (catalog, catalog_path);
 		g_free (catalog_path);
 
@@ -476,6 +509,8 @@ initialize_data (void)
 
 		view_comline_catalog = TRUE;
 	}
+	else
+		reset_command_line_catalog ();
 
 	g_free (current_dir);
 }
@@ -487,7 +522,7 @@ release_data (void)
 {
 #ifdef HAVE_GTKUNIQUE
 	if (gth_application != NULL)
-                g_object_unref (gth_application);
+		g_object_unref (gth_application);
 #endif /* HAVE_GTKUNIQUE */
 
 	free_icon_pixbufs ();
@@ -620,12 +655,12 @@ show_grabbing_focus (GtkWidget *new_window)
 #ifdef HAVE_GTKUNIQUE
 static GtkUniqueResponse
 application_message_cb (GtkUniqueApp     *app,
-                        GtkUniqueCommand  command,
-                        const gchar      *command_data,
-                        const gchar      *startup_id,
-                        GdkScreen        *screen,
-                        guint             workspace,
-                        gpointer          user_data)
+			GtkUniqueCommand  command,
+			const gchar      *command_data,
+			const gchar      *startup_id,
+			GdkScreen        *screen,
+			guint             workspace,
+			gpointer          user_data)
 {
 	char **tokens;
 	char *sub_command;
@@ -642,10 +677,10 @@ application_message_cb (GtkUniqueApp     *app,
 		uri = NULL;
 
       	if (strcmp (sub_command, "browser") == 0)
-        	show_grabbing_focus (gth_browser_new (uri));
+		show_grabbing_focus (gth_browser_new (uri));
 
 	else if (strcmp (sub_command, "viewer") == 0)
-        	show_grabbing_focus (gth_viewer_new (uri));
+		show_grabbing_focus (gth_viewer_new (uri));
 
       	else if (strcmp (sub_command, "load_image") == 0) {
 		if (UseViewer) {
@@ -667,7 +702,7 @@ application_message_cb (GtkUniqueApp     *app,
 		}
       	}
       	else if (strcmp (sub_command, "import") == 0)
-        	gth_browser_activate_action_file_camera_import (NULL, NULL);
+		gth_browser_activate_action_file_camera_import (NULL, NULL);
 
 	return GTK_UNIQUE_RESPONSE_OK;
 }
@@ -687,8 +722,8 @@ prepare_app (void)
 
 	if (! use_factory)
 		g_signal_connect (gth_application, "message",
-                		  G_CALLBACK (application_message_cb),
-                          	  NULL);
+				  G_CALLBACK (application_message_cb),
+				    NULL);
 #endif /* HAVE_GTKUNIQUE */
 
 	if (session_is_restored ()) {
@@ -724,24 +759,17 @@ prepare_app (void)
 		}
 
 	} else if (view_comline_catalog) {
-		char *tmp;
-		char *catalog_name;
 		char *catalog_path;
 		char *catalog_uri;
 
 		ViewFirstImage = TRUE;
 		HideSidebar = TRUE;
 
-		tmp = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
-		catalog_name = gnome_vfs_escape_string (tmp);
-		g_free (tmp);
-
-		catalog_path = get_catalog_full_path (catalog_name);
+		catalog_path = get_command_line_catalog_path ();
 		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
 
 		open_browser_window (catalog_uri, TRUE, use_factory);
 
-		g_free (catalog_name);
 		g_free (catalog_path);
 		g_free (catalog_uri);
 	}
@@ -775,9 +803,9 @@ main (int   argc,
 	textdomain (GETTEXT_PACKAGE);
 
 	description = g_strdup_printf ("- %s", _("View and organize your images"));
-        context = g_option_context_new (description);
-        g_free (description);
-        g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	context = g_option_context_new (description);
+	g_free (description);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 
 	program = gnome_program_init ("gthumb", VERSION,
 				      LIBGNOMEUI_MODULE,
@@ -983,20 +1011,7 @@ get_fs_icon (IconName icon_name,
 	GdkPixbuf *pixbuf = NULL;
 	gboolean   scale = TRUE;
 
-	/*
-	if (icon_pixbuf[icon_name] != NULL) {
-		g_object_ref (icon_pixbuf[icon_name]);
-		return icon_pixbuf[icon_name];
-	}
-
-	icon_pixbuf[icon_name] = create_pixbuf (icon_theme,
-						icon_mime_name[icon_name],
-						icon_size);
-
-	return icon_pixbuf[icon_name];
-	*/
-
-        if (icon_pixbuf[icon_name] == NULL) {
+	if (icon_pixbuf[icon_name] == NULL) {
 		GtkIconInfo         *icon_info = NULL;
 
 		icon_info = gtk_icon_theme_lookup_icon (icon_theme,
@@ -1005,16 +1020,16 @@ get_fs_icon (IconName icon_name,
 							0);
 
 		if (icon_info == NULL) {
-                	icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1,
-                                                                             dir_16_rgba,
-                                                                             FALSE,
-                                                                             NULL);
-                        scale = FALSE;
-                } else {
+			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1,
+									     dir_16_rgba,
+									     FALSE,
+									     NULL);
+			scale = FALSE;
+		} else {
 			icon_pixbuf[icon_name] = gtk_icon_info_load_icon (icon_info, NULL);
 			gtk_icon_info_free (icon_info);
-                }
-        }
+		}
+	}
 
 	/* Scale keeping aspect ratio. */
 
@@ -1056,8 +1071,8 @@ get_folder_pixbuf_size_for_list (GtkWidget *widget)
 	int icon_width, icon_height;
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           GTK_ICON_SIZE_SMALL_TOOLBAR,
-                                           &icon_width, &icon_height);
+					   GTK_ICON_SIZE_SMALL_TOOLBAR,
+					   &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
 
@@ -1068,8 +1083,8 @@ get_folder_pixbuf_size_for_menu (GtkWidget *widget)
 	int icon_width, icon_height;
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           GTK_ICON_SIZE_MENU,
-                                           &icon_width, &icon_height);
+					   GTK_ICON_SIZE_MENU,
+					   &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
 
@@ -1080,8 +1095,6 @@ folder_is_film (const char *folder)
 	CommentData *cdata;
 	gboolean     film = FALSE;
 
-	folder = remove_scheme_from_uri (folder);
-
 	cdata = comments_load_comment (folder, FALSE);
 	if (cdata != NULL) {
 		int i;
